@@ -47,7 +47,7 @@ static char *progname = "ser2net-control";
 
 DEFINE_LOCK_INIT(static, cntlr_lock)
 static struct addrinfo *cntrl_ai;
-static int *acceptfds;	/* The file descriptor for the accept port. */
+static struct opensocks *acceptfds;/* File descriptors for the accept port. */
 static unsigned int nr_acceptfds;
 static waiter_t *accept_waiter;
 
@@ -286,7 +286,7 @@ controller_outputf(struct controller_info *cntlr, const char *str, ...)
 
 void controller_outs(struct controller_info *cntlr, char *s)
 {
-  controller_output (cntlr, s, strlen(s));
+    controller_output (cntlr, s, strlen(s));
 }
 
 
@@ -349,10 +349,10 @@ static char *help_str =
 "       DTRHI, DTRLO, RTSHI, and RTSLO.\r\n"
 "setportenable <tcp port> <enable state> - Sets the port operation state.\r\n"
 "       Valid states are:\r\n"
-"         off - The TCP port is shut down\r\n"
-"         raw - The TCP port is up and all I/O is transferred\r\n"
-"         rawlp - The TCP port is up and the input is transferred to dev\r\n"
-"         telnet - The TCP port is up and the telnet negotiation protocol\r\n"
+"         off - The port is shut down\r\n"
+"         raw - The port is up and all I/O is transferred\r\n"
+"         rawlp - The port is up and the input is transferred to dev\r\n"
+"         telnet - The port is up and the telnet negotiation protocol\r\n"
 "                  runs on the port.\r\n";
 
 /* Process a line of input.  This scans for commands, reads any
@@ -618,7 +618,7 @@ handle_tcp_fd_read(int fd, void *data)
 	    i = -1;
 	} else {
 	    /* It's a normal character, just echo it. */
-	    controller_outs(cntlr, (char *) &(cntlr->inbuf[i]));
+	    controller_output(cntlr, (char *) &(cntlr->inbuf[i]), 1);
 	}
     }
  out_unlock:
@@ -844,6 +844,7 @@ int
 controller_init(char *controller_port)
 {
     int rv;
+    bool is_port_set;
 
     if (!controller_shutdown_waiter) {
 	controller_shutdown_waiter = alloc_waiter();
@@ -851,7 +852,7 @@ controller_init(char *controller_port)
 	    return ENOMEM;
     }
 
-    rv = scan_tcp_port(controller_port, &cntrl_ai);
+    rv = scan_network_port(controller_port, &cntrl_ai, NULL, &is_port_set);
     if (rv) {
 	if (rv == EINVAL)
 	    return CONTROLLER_INVALID_TCP_SPEC;
@@ -860,6 +861,8 @@ controller_init(char *controller_port)
 	else
 	    return -1;
     }
+    if (!is_port_set)
+	return CONTROLLER_INVALID_TCP_SPEC;
 
     if (!accept_waiter) {
 	accept_waiter = alloc_waiter();
@@ -869,7 +872,7 @@ controller_init(char *controller_port)
 	}
     }
 
-    acceptfds = open_socket(cntrl_ai, handle_accept_port_read, NULL,
+    acceptfds = open_socket(cntrl_ai, handle_accept_port_read, NULL, NULL,
 			    &nr_acceptfds, controller_accept_fd_cleared);
     if (acceptfds == NULL) {
 	freeaddrinfo(cntrl_ai);
@@ -888,9 +891,9 @@ controller_shutdown(void)
     if (acceptfds == NULL)
 	return;
     for (i = 0; i < nr_acceptfds; i++) {
-	sel_clear_fd_handlers(ser2net_sel, acceptfds[i]);
+	sel_clear_fd_handlers(ser2net_sel, acceptfds[i].fd);
 	wait_for_waiter(accept_waiter);
-	close(acceptfds[i]);
+	close(acceptfds[i].fd);
     }
     free(acceptfds);
     freeaddrinfo(cntrl_ai);
