@@ -534,13 +534,13 @@ devconfig(struct devcfg_data *d, struct absout *eout, const char *instr,
 	return -1;
     }
 
-    pos = strtok_r(str, ", \t", &strtok_data);
+    pos = strtok_r(str, " \t", &strtok_data);
     while (pos != NULL) {
 	if ((val = speedstr_to_speed(pos)) != -1) {
 	    rv = set_termios_from_speed(d, termctl, val);
 	    if (rv) {
 		eout->out(eout, "Invalid baud rate: %d\n", d->default_bps);
-		return -1;
+		goto out;
 	    }
 	} else if (strcmp(pos, "1STOPBIT") == 0) {
 	    termctl->c_cflag &= ~(CSTOPB);
@@ -556,7 +556,7 @@ devconfig(struct devcfg_data *d, struct absout *eout, const char *instr,
 	    set_termios_datasize(d, termctl, 7);
 	} else if (strcmp(pos, "8DATABITS") == 0) {
 	    set_termios_datasize(d, termctl, 8);
-	} else if ((val = lookup_parity(pos)) == 0) {
+	} else if ((val = lookup_parity(pos)) != -1) {
 	    set_termios_parity(d, termctl, val);
         } else if (strcmp(pos, "XONXOFF") == 0) {
 	    set_termios_xonoff(termctl, 1);
@@ -583,7 +583,7 @@ devconfig(struct devcfg_data *d, struct absout *eout, const char *instr,
 		goto out;
 	}
 
-	pos = strtok_r(NULL, ", \t", &strtok_data);
+	pos = strtok_r(NULL, " \t", &strtok_data);
     }
 
 #if HAVE_DECL_TIOCSRS485
