@@ -515,7 +515,7 @@ lookup_parity(const char *str)
     return lookup_enum(parity_enums, str, -1);
 }
 
-enum default_type { DEFAULT_INT, DEFAULT_BOOL, DEFAULT_ENUM };
+enum default_type { DEFAULT_INT, DEFAULT_BOOL, DEFAULT_ENUM, DEFAULT_STR };
 
 struct default_data
 {
@@ -526,10 +526,13 @@ struct default_data
     struct enum_val *enums;
     union {
 	int intval;
+	char *strval;
     } val;
     union {
 	int intval;
+	const char *strval;
     } def;
+    const char *altname;
 };
 
 struct default_data defaults[] = {
@@ -557,10 +560,12 @@ struct default_data defaults[] = {
 					.def.intval = 1000 },
     { "chardelay-max",	DEFAULT_INT,	.min = 1, .max = 1000000,
 					.def.intval = 20000 },
-    { "dev-to-tcp-bufsize", DEFAULT_INT,.min = 1, .max = 65536,
-					.def.intval = PORT_BUFSIZE },
-    { "tcp-to-dev-bufsize", DEFAULT_INT,.min = 1, .max = 65536,
-					.def.intval = PORT_BUFSIZE },
+    { "dev-to-net-bufsize", DEFAULT_INT,.min = 1, .max = 65536,
+					.def.intval = PORT_BUFSIZE,
+					.altname = "dev-to-tcp-bufsize" },
+    { "net-to-dev-bufsize", DEFAULT_INT,.min = 1, .max = 65536,
+					.def.intval = PORT_BUFSIZE,
+					.altname = "tcp-to-dev-bufsize" },
     { "max-connections", DEFAULT_INT,	.min=1, .max=65536,
 					.def.intval = 1 },
 #ifdef HAVE_OPENIPMI
@@ -575,6 +580,7 @@ struct default_data defaults[] = {
 				   .def.intval = ipmi_sol_serial_alerts_fail },
     { "deassert_CTS_DCD_DSR_on_connect", DEFAULT_BOOL, .def.intval = 0 },
 #endif
+    { "remaddr",	DEFAULT_STR,	.def.strval = "" },
     { NULL }
 };
 
@@ -585,27 +591,58 @@ setup_defaults(void)
     int i;
 
     for (i = 0; defaults[i].name; i++) {
-	defaults[i].val.intval = defaults[i].def.intval;
+	if (defaults[i].type == DEFAULT_STR) {
+	    if (defaults[i].val.strval) {
+		free(defaults[i].val.strval);
+		defaults[i].val.strval = NULL;
+	    }
+	} else {
+	    defaults[i].val.intval = defaults[i].def.intval;
+	}
     }
 }
 
+static int cmp_default_name(struct default_data *def, const char *name)
+{
+    return (strcmp(def->name, name) == 0 ||
+	    (def->altname && strcmp(def->altname, name) == 0));
+}
+
 int
 find_default_int(const char *name)
 {
     int i;
 
     for (i = 0; defaults[i].name; i++) {
-	if (strcmp(defaults[i].name, name) == 0)
+	if (cmp_default_name(&defaults[i], name) &&
+			defaults[i].type != DEFAULT_STR)
 	    return defaults[i].val.intval;
     }
     abort();
 }
 
+char *
+find_default_str(const char *name)
+{
+    int i;
+
+    for (i = 0; defaults[i].name; i++) {
+	if (cmp_default_name(&defaults[i], name) &&
+			defaults[i].type == DEFAULT_STR) {
+	    const char *s = defaults[i].val.strval;
+	    if (!s)
+		s = defaults[i].def.strval;
+	    return strdup(s);
+	}
+    }
+    abort();
+}
+
 static void
 handle_new_default(const char *name, const char *str)
 {
     int i, val, len;
-    char *end;
+    char *end, *sval;
     const char *s;
 
     while (isspace(*str))
@@ -620,7 +657,7 @@ handle_new_default(const char *name, const char *str)
     len = s - str;
 
     for (i = 0; defaults[i].name; i++) {
-	if (strcmp(defaults[i].name, name) != 0)
+	if (!cmp_default_name(&defaults[i], name))
 	    continue;
 
 	switch (defaults[i].type) {
@@ -661,9 +698,84 @@ handle_new_default(const char *name, const char *str)
 	    }
 	    defaults[i].val.intval = val;
 	    break;
+
+	case DEFAULT_STR:
+	    sval = strdup(str);
+	    if (!sval) {
+		syslog(LOG_ERR, "Out of memory processing default string on"
+		       " line %d", lineno);
+		return;
+	    }
+	    if (defaults[i].val.strval)
+		free(defaults[i].val.strval);
+	    defaults[i].val.strval = sval;
+	    break;
 	}
 	return;
     }
+
+    syslog(LOG_ERR, "unknown default name '%s' on %d", name, lineno);
+}
+
+/*
+ * This rather complicated variable is used to scan the string for
+ * ":off", ":telnet", ":raw:", or ":rawlp:".  It's a basic state
+ * machine where if a character in the first string "c" matches the
+ * current character, you go to the state machine index in the
+ * corresponding location giving by the character in string "next".
+ * So ":raw:" would see the ":" in state zero and go to state 1.  Then
+ * the "r" in state 1 would go to state 7.  Then "a" in state 7 would
+ * go to state 8, "w" to state name.  The ":" has a zero, that means a
+ * match was found.
+ *
+ * This is nasty, but it allows there to be ":" characters in the
+ * portnum so that IPV6 addresses can be specified.
+ */
+static struct {
+    char *c;
+    char *next;
+} scanstate[] = {
+    { ":",   "\x01" },		/* 0x00 */
+    { "tro", "\x02\x07\x0b" },	/* 0x01 */
+    { "e",   "\x03" },		/* 0x02 */
+    { "l",   "\x04" },		/* 0x03 */
+    { "n",   "\x05" },		/* 0x04 */
+    { "e",   "\x06" },		/* 0x05 */
+    { "t",   "\x0d" },		/* 0x06 */
+    { "a",   "\x08" },		/* 0x07 */
+    { "w",   "\x09" },		/* 0x09 */
+    { ":l",  "\x00\x0a" },	/* 0x09 */
+    { "p",   "\x0d" },		/* 0x0a */
+    { "f",   "\x0c" },		/* 0x0b */
+    { "f",   "\x0d" },		/* 0x0c */
+    { ":",   "\x00" }		/* 0x0d */
+};
+
+static char *
+scan_for_state(char *str)
+{
+    int s = 0;
+    char *b = str;
+
+    for (; *str; str++) {
+	int i;
+
+	for (i = 0; scanstate[s].c[i]; i++) {
+	    if (scanstate[s].c[i] == *str)
+		break;
+	}
+
+	if (scanstate[s].c[i]) {
+	    s = scanstate[s].next[i];
+	    if (s == 0)
+		return b;
+	} else {
+	    s = 0;
+	    b = str + 1;
+	}
+    }
+
+    return NULL;
 }
 
 int
@@ -833,19 +945,24 @@ handle_config_line(char *inbuf, int len)
 	goto out;
     }
 
-    portnum = strtok_r(inbuf, ":", &strtok_data);
-    if (portnum == NULL) {
-	/* An empty line is ok. */
-	goto out;
-    }
-
-    state = strtok_r(NULL, ":", &strtok_data);
-    if (state == NULL) {
+    /* Scan for the state. */
+    state = scan_for_state(inbuf);
+    if (!state) {
 	syslog(LOG_ERR, "No state given on line %d", lineno);
 	goto out;
     }
 
-    timeout = strtok_r(NULL, ":", &strtok_data);
+    /* Everything before the state is the port number. */
+    portnum = inbuf;
+    *state = '\0';
+    state++;
+
+    /* Terminate the state. */
+    inbuf = strchr(state, ':'); /* ":" must be there if scan_for_state works */
+    *inbuf = '\0';
+    inbuf++;
+
+    timeout = strtok_r(inbuf, ":", &strtok_data);
     if (timeout == NULL) {
 	syslog(LOG_ERR, "No timeout given on line %d", lineno);
 	goto out;
@@ -857,7 +974,7 @@ handle_config_line(char *inbuf, int len)
 	goto out;
     }
 
-    devcfg = strtok_r(NULL, ":", &strtok_data);
+    devcfg = strtok_r(NULL, "", &strtok_data);
     if (devcfg == NULL) {
 	/* An empty device config is ok. */
 	devcfg = "";
