@@ -746,6 +746,9 @@ solconfig(struct solcfg_data *d, struct absout *eout, const char *instr,
     } else if (rv) {
 	eout->out(eout, "Invalid SOL string: %s", d->io->devname + 4);
 	return -1;
+    } else if (argc == 0) {
+	eout->out(eout, "No SOL string given");
+	return -1;
     }
 
     rv = ipmi_parse_args2(&curr_arg, argc, argv, &args);
@@ -770,7 +773,7 @@ solconfig(struct solcfg_data *d, struct absout *eout, const char *instr,
 	goto out;
     }
 
-    pos = strtok_r(str, ", \t", &strtok_data);
+    pos = strtok_r(str, " \t", &strtok_data);
     while (pos != NULL) {
 	if (strcmp(pos, "9600") == 0) {
 	    d->speed = IPMI_SOL_BIT_RATE_9600;
@@ -823,7 +826,7 @@ solconfig(struct solcfg_data *d, struct absout *eout, const char *instr,
 		goto out;
 	}
 
-	pos = strtok_r(NULL, ", \t", &strtok_data);
+	pos = strtok_r(NULL, " \t", &strtok_data);
     }
 
  out:
