@@ -32,6 +32,17 @@
 #include "selector.h"
 #include "locking.h"
 
+int
+strisallzero(char *str)
+{
+    if (*str == '\0')
+	return 0;
+
+    while (*str == '0')
+	str++;
+    return *str == '\0';
+}
+
 /* Scan for a positive integer, and return it.  Return -1 if the
    integer was invalid. */
 int
@@ -63,17 +74,38 @@ scan_int(char *str)
     return rv;
 }
 
-/* Scan for a TCP port in the form "[hostname,]x", where the optional
+/* Scan for a network port in the form "[hostname,]x", where the optional
    first part is a resolvable hostname, an IPv4 octet, or an IPv6 address.
    In the absence of a host specification, a wildcard address is used.
    The mandatory second part is the port number or a service name. */
 int
-scan_tcp_port(char *str, struct addrinfo **rai)
+scan_network_port(const char *str, struct addrinfo **rai, bool *is_dgram,
+		  bool *is_port_set)
 {
     char *strtok_data, *strtok_buffer;
     char *ip;
     char *port;
     struct addrinfo hints, *ai;
+    int family = AF_UNSPEC;
+    int socktype = SOCK_STREAM;
+
+    if (strncmp(str, "ipv4,", 5) == 0) {
+	family = AF_INET;
+	str += 5;
+    } else if (strncmp(str, "ipv6,", 5) == 0) {
+	family = AF_INET6;
+	str += 5;
+    }
+
+    if (strncmp(str, "tcp,", 4) == 0) {
+	str += 4;
+    } else if (strncmp(str, "udp,", 4) == 0) {
+	/* Only allow UDP if asked for. */
+	if (!is_dgram)
+	    return EINVAL;
+	socktype = SOCK_DGRAM;
+	str += 4;
+    }
 
     strtok_buffer = strdup(str);
     if (!strtok_buffer)
@@ -88,13 +120,19 @@ scan_tcp_port(char *str, struct addrinfo **rai)
 
     memset(&hints, 0, sizeof(hints));
     hints.ai_flags = AI_PASSIVE;
-    hints.ai_family = AF_UNSPEC;
-    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_family = family;
+    hints.ai_socktype = socktype;
     if (getaddrinfo(ip, port, &hints, &ai)) {
 	free(strtok_buffer);
 	return EINVAL;
     }
 
+    if (is_dgram)
+	*is_dgram = socktype == SOCK_DGRAM;
+
+    if (is_port_set)
+	*is_port_set = !strisallzero(port);
+
     free(strtok_buffer);
     if (*rai)
 	freeaddrinfo(*rai);
@@ -102,6 +140,47 @@ scan_tcp_port(char *str, struct addrinfo **rai)
     return 0;
 }
 
+bool
+sockaddr_equal(struct sockaddr *a1, socklen_t l1,
+	       struct sockaddr *a2, socklen_t l2,
+	       bool compare_ports)
+{
+    if (l1 != l2)
+	return false;
+    if (a1->sa_family != a2->sa_family)
+	return false;
+    switch (a1->sa_family) {
+    case AF_INET:
+	{
+	    struct sockaddr_in *s1 = (struct sockaddr_in *) a1;
+	    struct sockaddr_in *s2 = (struct sockaddr_in *) a2;
+	    if (compare_ports && s1->sin_port != s2->sin_port)
+		return false;
+	    if (s1->sin_addr.s_addr != s2->sin_addr.s_addr)
+		return false;
+	}
+	break;
+
+    case AF_INET6:
+	{
+	    struct sockaddr_in6 *s1 = (struct sockaddr_in6 *) a1;
+	    struct sockaddr_in6 *s2 = (struct sockaddr_in6 *) a2;
+	    if (compare_ports && s1->sin6_port != s2->sin6_port)
+		return false;
+	    if (memcmp(s1->sin6_addr.s6_addr, s2->sin6_addr.s6_addr,
+		       sizeof(s1->sin6_addr.s6_addr)) != 0)
+		return false;
+	}
+	break;
+
+    default:
+	/* Unknown family. */
+	return false;
+    }
+
+    return true;
+}
+
 void
 check_ipv6_only(int family, struct sockaddr *addr, int fd)
 {
@@ -114,21 +193,25 @@ check_ipv6_only(int family, struct sockaddr *addr, int fd)
     }
 }
 
-int *
-open_socket(struct addrinfo *ai, void (*readhndlr)(int, void *), void *data,
+struct opensocks *
+open_socket(struct addrinfo *ai, void (*readhndlr)(int, void *),
+	    void (*writehndlr)(int, void *), void *data,
 	    unsigned int *nr_fds, void (*fd_handler_cleared)(int, void *))
 {
     struct addrinfo *rp;
     int optval = 1;
     int family = AF_INET6; /* Try IPV6 first, then IPV4. */
-    int *fds;
+    struct opensocks *fds;
     unsigned int curr_fd = 0;
     unsigned int max_fds = 0;
 
     for (rp = ai; rp != NULL; rp = rp->ai_next)
 	max_fds++;
 
-    fds = malloc(sizeof(int) * max_fds);
+    if (max_fds == 0)
+	return NULL;
+
+    fds = malloc(sizeof(*fds) * max_fds);
     if (!fds)
 	return NULL;
 
@@ -137,34 +220,37 @@ open_socket(struct addrinfo *ai, void (*readhndlr)(int, void *), void *data,
 	if (family != rp->ai_family)
 	    continue;
 
-	fds[curr_fd] = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
-	if (fds[curr_fd] == -1)
+	fds[curr_fd].fd = socket(rp->ai_family, rp->ai_socktype,
+				 rp->ai_protocol);
+	if (fds[curr_fd].fd == -1)
 	    continue;
 
-	if (fcntl(fds[curr_fd], F_SETFL, O_NONBLOCK) == -1)
+	fds[curr_fd].family = rp->ai_family;
+
+	if (fcntl(fds[curr_fd].fd, F_SETFL, O_NONBLOCK) == -1)
 	    goto next;
 
-	if (setsockopt(fds[curr_fd], SOL_SOCKET, SO_REUSEADDR,
+	if (setsockopt(fds[curr_fd].fd, SOL_SOCKET, SO_REUSEADDR,
 		       (void *)&optval, sizeof(optval)) == -1)
 	    goto next;
 
-	check_ipv6_only(rp->ai_family, rp->ai_addr, fds[curr_fd]);
+	check_ipv6_only(rp->ai_family, rp->ai_addr, fds[curr_fd].fd);
 
-	if (bind(fds[curr_fd], rp->ai_addr, rp->ai_addrlen) != 0)
+	if (bind(fds[curr_fd].fd, rp->ai_addr, rp->ai_addrlen) != 0)
 	    goto next;
 
-	if (listen(fds[curr_fd], 1) != 0)
+	if (rp->ai_socktype == SOCK_STREAM && listen(fds[curr_fd].fd, 1) != 0)
 	    goto next;
 
-	sel_set_fd_handlers(ser2net_sel, fds[curr_fd], data,
-			    readhndlr, NULL, NULL, fd_handler_cleared);
-	sel_set_fd_read_handler(ser2net_sel, fds[curr_fd],
+	sel_set_fd_handlers(ser2net_sel, fds[curr_fd].fd, data,
+			    readhndlr, writehndlr, NULL, fd_handler_cleared);
+	sel_set_fd_read_handler(ser2net_sel, fds[curr_fd].fd,
 				SEL_FD_HANDLER_ENABLED);
 	curr_fd++;
 	continue;
 
       next:
-	close(fds[curr_fd]);
+	close(fds[curr_fd].fd);
     }
     if (family == AF_INET6) {
 	family = AF_INET;
@@ -212,6 +298,8 @@ void str_to_argv_free(int argc, char **argv)
 {
     int i;
 
+    if (!argv)
+	return;
     for (i = 0; i < argc; i++) {
 	if (argv[i])
 	    free(argv[i]);
@@ -284,7 +372,7 @@ int str_to_argv(const char *s, int *r_argc, char ***r_argv, char *seps)
     char *parm = NULL;
     size_t parmlen = 0;
     size_t parmpos = 0;
-    int rv;
+    int rv = 0;
     int c = 0;
 
     if (!seps)
