@@ -24,16 +24,36 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netdb.h>
+#include <stdbool.h>
+
+/*
+ * Compare two sockaddr structure and return TRUE if they are equal
+ * and FALSE if not.  Only works for AF_INET4 and AF_INET6.
+ * If a2->sin_port is zero, then the port comparison is ignored.
+ */
+bool sockaddr_equal(struct sockaddr *a1, socklen_t l1,
+		    struct sockaddr *a2, socklen_t l2,
+		    bool compare_ports);
+
+/* Returns true if the string is a numeric zero, false if not. */
+int strisallzero(char *str);
 
 /* Scan for a positive integer, and return it.  Return -1 if the
    integer was invalid.  Spaces are not handled. */
 int scan_int(char *str);
 
-/* Scan for a TCP port in the form "[hostname,]x", where the optional
+/* Scan for a network port in the form "[hostname,]x", where the optional
  * first part is a resolvable hostname, an IPv4 octet, or an IPv6 address.
  * In the absence of a host specification, a wildcard address is used.
  * The mandatory second part is the port number or a service name. */
-int scan_tcp_port(char *str, struct addrinfo **ai);
+int scan_network_port(const char *str, struct addrinfo **ai, bool *is_dgram,
+		      bool *is_port_set);
+
+struct opensocks
+{
+    int fd;
+    int family;
+};
 
 /*
  * Open a set of sockets given the addrinfo list, one per address.
@@ -45,9 +65,11 @@ int scan_tcp_port(char *str, struct addrinfo **ai);
  * Also, open IPV6 addresses first.  This way, addresses in shared
  * namespaces (like IPV4 and IPV6 on INADDR6_ANY) will work properly
  */
-int *open_socket(struct addrinfo *ai, void (*readhndlr)(int, void *),
-		 void *data, unsigned int *nr_fds,
-		 void (*fd_handler_cleared)(int, void *));
+struct opensocks *
+open_socket(struct addrinfo *ai, void (*readhndlr)(int, void *),
+	    void (*writehndlr)(int, void *), void *data,
+	    unsigned int *nr_fds,
+	    void (*fd_handler_cleared)(int, void *));
 
 /*
  * Search for a banner/open/close string by name.  Note that the
@@ -90,6 +112,10 @@ enum parity_vals lookup_parity(const char *str);
 /* Return the default int value for the given name. */
 int find_default_int(const char *name);
 
+/* Return the default string value for the given name.  Return NULL if
+   out of memory.  The returned value must be freed. */
+char *find_default_str(const char *name);
+
 /* Separate out a string into an argv array, returning the argc/argv
    values given.  Returns -ENOMEM when out of memory or -EINVAL if
    there is something wrong with the string.  seps is a list of
@@ -108,4 +134,9 @@ void free_waiter(waiter_t *waiter);
 void wait_for_waiter(waiter_t *waiter);
 void wake_waiter(waiter_t *waiter);
 
+struct absout {
+    int (*out)(struct absout *e, const char *str, ...);
+    void *data;
+};
+
 #endif /* UTILS */
